/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSRuleList.h"
#include "CSSStyleDeclaration.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMSelection.h"
#include "DOMWindow.h"
#include "Database.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomCSSRuleList.h"
#include "gdom/GdomCSSStyleDeclaration.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomDOMSelection.h"
#include "gdom/GdomDOMWindow.h"
#include "gdom/GdomDatabase.h"
#include "gdom/GdomElement.h"
#include "gdom/PrivateGdomCSSRuleList.h"
#include "gdom/PrivateGdomCSSStyleDeclaration.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDOMSelection.h"
#include "gdom/PrivateGdomDOMWindow.h"
#include "gdom/PrivateGdomDatabase.h"
#include "gdom/PrivateGdomElement.h"
#include "gdom/PrivateIntGdomCSSRuleList.h"
#include "gdom/PrivateIntGdomCSSStyleDeclaration.h"
#include "gdom/PrivateIntGdomDOMSelection.h"
#include "gdom/PrivateIntGdomDOMWindow.h"
#include "gdom/PrivateIntGdomDatabase.h"
#include "gdom/PrivateIntGdomElement.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::DOMWindow* obj)
{
    return WebCore::cacheDOMObject<WebCore::DOMWindow, GDOMDOMWindow>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMDOMWindow::GDOMDOMWindow(PassRefPtr<WebCore::DOMWindow> impl)
    :m_impl(impl)
{
}

GDOMDOMWindow::~GDOMDOMWindow()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}

GdomDOMSelection *
gdom_dom_window_get_selection (GdomDOMWindow *thiz)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_val_if_fail (item, 0);
    PassRefPtr<WebCore::DOMSelection> g_res = WTF::getPtr(item->getSelection());
    GdomDOMSelection * res = static_cast<GdomDOMSelection *>(WebCore::toGDOM(g_res.get()));
    return res;

}

void
gdom_dom_window_focus (GdomDOMWindow *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->focus();

}

void
gdom_dom_window_blur (GdomDOMWindow *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->blur();

}

void
gdom_dom_window_close (GdomDOMWindow *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->close();

}

void
gdom_dom_window_print (GdomDOMWindow *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->print();

}

void
gdom_dom_window_stop (GdomDOMWindow *thiz)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->stop();

}

void
gdom_dom_window_alert (GdomDOMWindow *thiz, gchar * message)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (message);
    WebCore::String _g_message = WebCore::String::fromUTF8(message);
    item->alert(_g_message);

}

gboolean
gdom_dom_window_confirm (GdomDOMWindow *thiz, gchar * message)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (message, 0);
    WebCore::String _g_message = WebCore::String::fromUTF8(message);
    gboolean res = item->confirm(_g_message);
    return res;

}

gchar *
gdom_dom_window_prompt (GdomDOMWindow *thiz, gchar * message, gchar * default_value)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (message, 0);
    g_return_val_if_fail (default_value, 0);
    WebCore::String _g_message = WebCore::String::fromUTF8(message);
    WebCore::String _g_default_value = WebCore::String::fromUTF8(default_value);
    gchar * res = _G_GET_UTF8(item->prompt(_g_message, _g_default_value));
    return res;

}

gboolean
gdom_dom_window_find (GdomDOMWindow *thiz, gchar * string, gboolean case_sensitive, gboolean backwards, gboolean wrap, gboolean whole_word, gboolean search_in_frames, gboolean show_dialog)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (string, 0);
    WebCore::String _g_string = WebCore::String::fromUTF8(string);
    gboolean res = item->find(_g_string, case_sensitive, backwards, wrap, whole_word, search_in_frames, show_dialog);
    return res;

}

void
gdom_dom_window_scroll_by (GdomDOMWindow *thiz, glong x, glong y)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->scrollBy(x, y);

}

void
gdom_dom_window_scroll_to (GdomDOMWindow *thiz, glong x, glong y)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->scrollTo(x, y);

}

void
gdom_dom_window_scroll (GdomDOMWindow *thiz, glong x, glong y)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->scroll(x, y);

}

void
gdom_dom_window_move_by (GdomDOMWindow *thiz, gfloat x, gfloat y)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->moveBy(x, y);

}

void
gdom_dom_window_move_to (GdomDOMWindow *thiz, gfloat x, gfloat y)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->moveTo(x, y);

}

void
gdom_dom_window_resize_by (GdomDOMWindow *thiz, gfloat x, gfloat y)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->resizeBy(x, y);

}

void
gdom_dom_window_resize_to (GdomDOMWindow *thiz, gfloat width, gfloat height)
{
    g_return_if_fail (thiz);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_if_fail (item);
    item->resizeTo(width, height);

}

GdomCSSStyleDeclaration *
gdom_dom_window_get_computed_style (GdomDOMWindow *thiz, GdomElement * element, gchar * pseudo_element)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (element, 0);
    g_return_val_if_fail (pseudo_element, 0);
    WebCore::Element * _g_element = (gdom_element_get_private(element));
    g_return_val_if_fail (_g_element, 0);
    WebCore::String _g_pseudo_element = WebCore::String::fromUTF8(pseudo_element);
    PassRefPtr<WebCore::CSSStyleDeclaration> g_res = WTF::getPtr(item->getComputedStyle(_g_element, _g_pseudo_element));
    GdomCSSStyleDeclaration * res = static_cast<GdomCSSStyleDeclaration *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomCSSRuleList *
gdom_dom_window_get_matched_css_rules (GdomDOMWindow *thiz, GdomElement * element, gchar * pseudo_element, gboolean author_only)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (element, 0);
    g_return_val_if_fail (pseudo_element, 0);
    WebCore::Element * _g_element = (gdom_element_get_private(element));
    g_return_val_if_fail (_g_element, 0);
    WebCore::String _g_pseudo_element = WebCore::String::fromUTF8(pseudo_element);
    PassRefPtr<WebCore::CSSRuleList> g_res = WTF::getPtr(item->getMatchedCSSRules(_g_element, _g_pseudo_element, author_only));
    GdomCSSRuleList * res = static_cast<GdomCSSRuleList *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomDatabase *
gdom_dom_window_open_database (GdomDOMWindow *thiz, gchar * name, gchar * version, gchar * display_name, gulong estimated_size)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::DOMWindow * item = (gdom_dom_window_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    g_return_val_if_fail (version, 0);
    g_return_val_if_fail (display_name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    WebCore::String _g_version = WebCore::String::fromUTF8(version);
    WebCore::String _g_display_name = WebCore::String::fromUTF8(display_name);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Database> g_res = WTF::getPtr(item->openDatabase(_g_name, _g_version, _g_display_name, estimated_size, ec));
    GdomDatabase * res = static_cast<GdomDatabase *>(WebCore::toGDOM(g_res.get()));
    return res;

}


/* XXX TODO: custom function gdom_dom_window_post_message */

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "DOMWindow.h"
#include <gdom/GdomDOMWindow.h>
#include <gdom/PrivateGdomDOMWindow.h>
#include <gdom/PrivateIntGdomDOMWindow.h>
#include <gdom/GdomDOMWindowprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomDOMWindow, gdom_dom_window, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomDOMWindowPrivate {
    class WebCore::DOMWindow *item;
};

#define GDOM_DOM_WINDOW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_DOM_WINDOW, GdomDOMWindowPrivate))
*/
G_END_DECLS
WebCore::DOMWindow *
gdom_dom_window_get_private(GdomDOMWindow *request)
{
    GDOMDOMWindow *res;
    res = static_cast<GDOMDOMWindow*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::DOMWindow*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_SCREEN,
    PROP_HISTORY,
    PROP_LOCATIONBAR,
    PROP_MENUBAR,
    PROP_PERSONALBAR,
    PROP_SCROLLBARS,
    PROP_STATUSBAR,
    PROP_TOOLBAR,
    PROP_NAVIGATOR,
    PROP_CLIENT_INFORMATION,
    PROP_LOCATION,
    PROP_FRAME_ELEMENT,
    PROP_OFFSCREEN_BUFFERING,
    PROP_OUTER_HEIGHT,
    PROP_OUTER_WIDTH,
    PROP_INNER_HEIGHT,
    PROP_INNER_WIDTH,
    PROP_SCREEN_X,
    PROP_SCREEN_Y,
    PROP_SCREEN_LEFT,
    PROP_SCREEN_TOP,
    PROP_SCROLL_X,
    PROP_SCROLL_Y,
    PROP_PAGE_X_OFFSET,
    PROP_PAGE_Y_OFFSET,
    PROP_CLOSED,
    PROP_LENGTH,
    PROP_NAME,
    PROP_STATUS,
    PROP_DEFAULT_STATUS,
    PROP_SELF,
    PROP_WINDOW,
    PROP_FRAMES,
    PROP_OPENER,
    PROP_PARENT,
    PROP_TOP,
    PROP_DOCUMENT,
    PROP_DEVICE_PIXEL_RATIO,
    PROP_APPLICATION_CACHE,
    PROP_SESSION_STORAGE,
    PROP_LOCAL_STORAGE,
    PROP_CONSOLE,
};


static void gdom_dom_window_finalize(GObject* object)
{
    /*GdomDOMWindow* request = GDOM_DOM_WINDOW(object);*/
    /*GdomDOMWindowPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_dom_window_parent_class)->finalize(object);
}

static void gdom_dom_window__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMWindow *self = GDOM_DOM_WINDOW(object);
    WebCore::DOMWindow * gself = (gdom_dom_window_get_private(self));
    switch (prop_id) {
     case PROP_NAME:
    {
         gself->setName(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_STATUS:
    {
         gself->setStatus(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
     case PROP_DEFAULT_STATUS:
    {
         gself->setDefaultStatus(WebCore::String::fromUTF8(g_value_get_string (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_window__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomDOMWindow *self = GDOM_DOM_WINDOW(object);
    WebCore::DOMWindow * gself = (gdom_dom_window_get_private(self));
    switch (prop_id) {
    case PROP_SCREEN:
    {
         PassRefPtr<WebCore::Screen> ptr = gself->screen();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_HISTORY:
    {
         PassRefPtr<WebCore::History> ptr = gself->history();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_LOCATIONBAR:
    {
         PassRefPtr<WebCore::BarInfo> ptr = gself->locationbar();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_MENUBAR:
    {
         PassRefPtr<WebCore::BarInfo> ptr = gself->menubar();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PERSONALBAR:
    {
         PassRefPtr<WebCore::BarInfo> ptr = gself->personalbar();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_SCROLLBARS:
    {
         PassRefPtr<WebCore::BarInfo> ptr = gself->scrollbars();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_STATUSBAR:
    {
         PassRefPtr<WebCore::BarInfo> ptr = gself->statusbar();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_TOOLBAR:
    {
         PassRefPtr<WebCore::BarInfo> ptr = gself->toolbar();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_NAVIGATOR:
    {
         PassRefPtr<WebCore::Navigator> ptr = gself->navigator();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_CLIENT_INFORMATION:
    {
         PassRefPtr<WebCore::Navigator> ptr = gself->clientInformation();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_LOCATION:
    {
         PassRefPtr<WebCore::Location> ptr = gself->location();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_FRAME_ELEMENT:
    {
         PassRefPtr<WebCore::Element> ptr = gself->frameElement();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_OFFSCREEN_BUFFERING:
    {
         g_value_set_boolean (value, gself->offscreenBuffering());
         break;
    }
    case PROP_OUTER_HEIGHT:
    {
         g_value_set_long (value, gself->outerHeight());
         break;
    }
    case PROP_OUTER_WIDTH:
    {
         g_value_set_long (value, gself->outerWidth());
         break;
    }
    case PROP_INNER_HEIGHT:
    {
         g_value_set_long (value, gself->innerHeight());
         break;
    }
    case PROP_INNER_WIDTH:
    {
         g_value_set_long (value, gself->innerWidth());
         break;
    }
    case PROP_SCREEN_X:
    {
         g_value_set_long (value, gself->screenX());
         break;
    }
    case PROP_SCREEN_Y:
    {
         g_value_set_long (value, gself->screenY());
         break;
    }
    case PROP_SCREEN_LEFT:
    {
         g_value_set_long (value, gself->screenLeft());
         break;
    }
    case PROP_SCREEN_TOP:
    {
         g_value_set_long (value, gself->screenTop());
         break;
    }
    case PROP_SCROLL_X:
    {
         g_value_set_long (value, gself->scrollX());
         break;
    }
    case PROP_SCROLL_Y:
    {
         g_value_set_long (value, gself->scrollY());
         break;
    }
    case PROP_PAGE_X_OFFSET:
    {
         g_value_set_long (value, gself->pageXOffset());
         break;
    }
    case PROP_PAGE_Y_OFFSET:
    {
         g_value_set_long (value, gself->pageYOffset());
         break;
    }
    case PROP_CLOSED:
    {
         g_value_set_boolean (value, gself->closed());
         break;
    }
    case PROP_LENGTH:
    {
         g_value_set_ulong (value, gself->length());
         break;
    }
    case PROP_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->name()));
         break;
    }
    case PROP_STATUS:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->status()));
         break;
    }
    case PROP_DEFAULT_STATUS:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->defaultStatus()));
         break;
    }
    case PROP_SELF:
    {
         PassRefPtr<WebCore::DOMWindow> ptr = gself->self();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_WINDOW:
    {
         PassRefPtr<WebCore::DOMWindow> ptr = gself->window();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_FRAMES:
    {
         PassRefPtr<WebCore::DOMWindow> ptr = gself->frames();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_OPENER:
    {
         PassRefPtr<WebCore::DOMWindow> ptr = gself->opener();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PARENT:
    {
         PassRefPtr<WebCore::DOMWindow> ptr = gself->parent();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_TOP:
    {
         PassRefPtr<WebCore::DOMWindow> ptr = gself->top();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_DOCUMENT:
    {
         PassRefPtr<WebCore::Document> ptr = gself->document();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_DEVICE_PIXEL_RATIO:
    {
         g_value_set_double (value, gself->devicePixelRatio());
         break;
    }
    case PROP_APPLICATION_CACHE:
    {
         PassRefPtr<WebCore::DOMApplicationCache> ptr = gself->applicationCache();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_SESSION_STORAGE:
    {
         PassRefPtr<WebCore::Storage> ptr = gself->sessionStorage();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_LOCAL_STORAGE:
    {
         PassRefPtr<WebCore::Storage> ptr = gself->localStorage();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_CONSOLE:
    {
         PassRefPtr<WebCore::Console> ptr = gself->console();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_dom_window_class_init(GdomDOMWindowClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_dom_window_finalize;
    g_klass->set_property = gdom_dom_window__set_property_fn;
    g_klass->get_property = gdom_dom_window__get_property_fn;

    g_object_class_install_property (g_klass, PROP_SCREEN,
            g_param_spec_object ("screen", /* name */
                "dom_window_screen", /* short description */
                "read-only  GdomScreen * DOMWindow.screen", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_SCREEN, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_HISTORY,
            g_param_spec_object ("history", /* name */
                "dom_window_history", /* short description */
                "read-only  GdomHistory * DOMWindow.history", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_HISTORY, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LOCATIONBAR,
            g_param_spec_object ("locationbar", /* name */
                "dom_window_locationbar", /* short description */
                "read-write  GdomBarInfo * DOMWindow.locationbar", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_BAR_INFO, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_MENUBAR,
            g_param_spec_object ("menubar", /* name */
                "dom_window_menubar", /* short description */
                "read-write  GdomBarInfo * DOMWindow.menubar", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_BAR_INFO, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PERSONALBAR,
            g_param_spec_object ("personalbar", /* name */
                "dom_window_personalbar", /* short description */
                "read-write  GdomBarInfo * DOMWindow.personalbar", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_BAR_INFO, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCROLLBARS,
            g_param_spec_object ("scrollbars", /* name */
                "dom_window_scrollbars", /* short description */
                "read-write  GdomBarInfo * DOMWindow.scrollbars", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_BAR_INFO, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_STATUSBAR,
            g_param_spec_object ("statusbar", /* name */
                "dom_window_statusbar", /* short description */
                "read-write  GdomBarInfo * DOMWindow.statusbar", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_BAR_INFO, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TOOLBAR,
            g_param_spec_object ("toolbar", /* name */
                "dom_window_toolbar", /* short description */
                "read-write  GdomBarInfo * DOMWindow.toolbar", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_BAR_INFO, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_NAVIGATOR,
            g_param_spec_object ("navigator", /* name */
                "dom_window_navigator", /* short description */
                "read-write  GdomNavigator * DOMWindow.navigator", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NAVIGATOR, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_CLIENT_INFORMATION,
            g_param_spec_object ("client_information", /* name */
                "dom_window_client_information", /* short description */
                "read-write  GdomNavigator * DOMWindow.client_information", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_NAVIGATOR, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_LOCATION,
            g_param_spec_object ("location", /* name */
                "dom_window_location", /* short description */
                "read-write  GdomLocation * DOMWindow.location", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_LOCATION, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_FRAME_ELEMENT,
            g_param_spec_object ("frame_element", /* name */
                "dom_window_frame_element", /* short description */
                "read-only  GdomElement * DOMWindow.frame_element", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSCREEN_BUFFERING,
            g_param_spec_boolean ("offscreen_buffering", /* name */
                "dom_window_offscreen_buffering", /* short description */
                "read-write  gboolean DOMWindow.offscreen_buffering", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_OUTER_HEIGHT,
            g_param_spec_long ("outer_height", /* name */
                "dom_window_outer_height", /* short description */
                "read-write  glong DOMWindow.outer_height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_OUTER_WIDTH,
            g_param_spec_long ("outer_width", /* name */
                "dom_window_outer_width", /* short description */
                "read-write  glong DOMWindow.outer_width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_INNER_HEIGHT,
            g_param_spec_long ("inner_height", /* name */
                "dom_window_inner_height", /* short description */
                "read-write  glong DOMWindow.inner_height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_INNER_WIDTH,
            g_param_spec_long ("inner_width", /* name */
                "dom_window_inner_width", /* short description */
                "read-write  glong DOMWindow.inner_width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCREEN_X,
            g_param_spec_long ("screen_x", /* name */
                "dom_window_screen_x", /* short description */
                "read-write  glong DOMWindow.screen_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCREEN_Y,
            g_param_spec_long ("screen_y", /* name */
                "dom_window_screen_y", /* short description */
                "read-write  glong DOMWindow.screen_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCREEN_LEFT,
            g_param_spec_long ("screen_left", /* name */
                "dom_window_screen_left", /* short description */
                "read-write  glong DOMWindow.screen_left", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCREEN_TOP,
            g_param_spec_long ("screen_top", /* name */
                "dom_window_screen_top", /* short description */
                "read-write  glong DOMWindow.screen_top", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCROLL_X,
            g_param_spec_long ("scroll_x", /* name */
                "dom_window_scroll_x", /* short description */
                "read-write  glong DOMWindow.scroll_x", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCROLL_Y,
            g_param_spec_long ("scroll_y", /* name */
                "dom_window_scroll_y", /* short description */
                "read-write  glong DOMWindow.scroll_y", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PAGE_X_OFFSET,
            g_param_spec_long ("page_x_offset", /* name */
                "dom_window_page_x_offset", /* short description */
                "read-only  glong DOMWindow.page_x_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PAGE_Y_OFFSET,
            g_param_spec_long ("page_y_offset", /* name */
                "dom_window_page_y_offset", /* short description */
                "read-only  glong DOMWindow.page_y_offset", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLOSED,
            g_param_spec_boolean ("closed", /* name */
                "dom_window_closed", /* short description */
                "read-only  gboolean DOMWindow.closed", /* longer - could do with some extra doc stuff here */
                 FALSE, /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LENGTH,
            g_param_spec_ulong ("length", /* name */
                "dom_window_length", /* short description */
                "read-write  gulong DOMWindow.length", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_NAME,
            g_param_spec_string ("name", /* name */
                "dom_window_name", /* short description */
                "read-write  gchar * DOMWindow.name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_STATUS,
            g_param_spec_string ("status", /* name */
                "dom_window_status", /* short description */
                "read-write  gchar * DOMWindow.status", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_DEFAULT_STATUS,
            g_param_spec_string ("default_status", /* name */
                "dom_window_default_status", /* short description */
                "read-write  gchar * DOMWindow.default_status", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SELF,
            g_param_spec_object ("self", /* name */
                "dom_window_self", /* short description */
                "read-write  GdomDOMWindow * DOMWindow.self", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_WINDOW, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_WINDOW,
            g_param_spec_object ("window", /* name */
                "dom_window_window", /* short description */
                "read-only  GdomDOMWindow * DOMWindow.window", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_WINDOW, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FRAMES,
            g_param_spec_object ("frames", /* name */
                "dom_window_frames", /* short description */
                "read-write  GdomDOMWindow * DOMWindow.frames", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_WINDOW, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_OPENER,
            g_param_spec_object ("opener", /* name */
                "dom_window_opener", /* short description */
                "read-write  GdomDOMWindow * DOMWindow.opener", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_WINDOW, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PARENT,
            g_param_spec_object ("parent", /* name */
                "dom_window_parent", /* short description */
                "read-write  GdomDOMWindow * DOMWindow.parent", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_WINDOW, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_TOP,
            g_param_spec_object ("top", /* name */
                "dom_window_top", /* short description */
                "read-write  GdomDOMWindow * DOMWindow.top", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_WINDOW, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_DOCUMENT,
            g_param_spec_object ("document", /* name */
                "dom_window_document", /* short description */
                "read-only  GdomDocument * DOMWindow.document", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOCUMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_DEVICE_PIXEL_RATIO,
            g_param_spec_double ("device_pixel_ratio", /* name */
                "dom_window_device_pixel_ratio", /* short description */
                "read-write  gdouble DOMWindow.device_pixel_ratio", /* longer - could do with some extra doc stuff here */
                 G_MINDOUBLE,        /* min */
                G_MAXDOUBLE, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_APPLICATION_CACHE,
            g_param_spec_object ("application_cache", /* name */
                "dom_window_application_cache", /* short description */
                "read-only  GdomDOMApplicationCache * DOMWindow.application_cache", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_DOM_APPLICATION_CACHE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SESSION_STORAGE,
            g_param_spec_object ("session_storage", /* name */
                "dom_window_session_storage", /* short description */
                "read-only  GdomStorage * DOMWindow.session_storage", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_STORAGE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LOCAL_STORAGE,
            g_param_spec_object ("local_storage", /* name */
                "dom_window_local_storage", /* short description */
                "read-only  GdomStorage * DOMWindow.local_storage", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_STORAGE, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CONSOLE,
            g_param_spec_object ("console", /* name */
                "dom_window_console", /* short description */
                "read-write  GdomConsole * DOMWindow.console", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CONSOLE, /* gobject type */
                 (GParamFlags)G_PARAM_READWRITE));



    event_signals[BROWSER_EVENT] = g_signal_new("browser-event",
        G_TYPE_FROM_CLASS(requestClass),
        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION |
                                           G_SIGNAL_DETAILED  ),
        0,
        NULL,
        NULL,
        webkit_marshal_VOID__OBJECT_BOOLEAN,
        G_TYPE_NONE, 2,
        GDOM_TYPE_EVENT, G_TYPE_BOOLEAN);


    /*g_type_class_remove_private(requestClass, sizeof(GdomDOMWindowPrivate));*/
}

static void gdom_dom_window_init(GdomDOMWindow* request)
{
    /*GdomDOMWindowPrivate* priv = GDOM_DOM_WINDOW_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

